import ts from 'typescript';
export function createCustomSys(cwd, virtualFileExtensions) {
    const sys = {
        ...ts.sys,
        getCurrentDirectory: () => cwd,
    };
    if (virtualFileExtensions.length === 0)
        return sys;
    return {
        ...sys,
        fileExists(path) {
            return ts.sys.fileExists(ensureRealFilePath(path, virtualFileExtensions));
        },
    };
}
function toRealFilePath(filePath) {
    return filePath.slice(0, -'.ts'.length);
}
export function isVirtualFilePath(filePath, extensions) {
    return extensions.some(extension => filePath.endsWith(`${extension}.ts`));
}
export function ensureRealFilePath(filePath, extensions) {
    return isVirtualFilePath(filePath, extensions) ? toRealFilePath(filePath) : filePath;
}
